module CbId = Misc.Id ()

type object_id = Oid.t
type callback_id = CbId.t

type _ ev = ..

module O = struct
  type t = unit ev
  let compare t t' =
    (*Log.debug
        (fun m -> m "comparing %s and %s"
           (Printexc.to_string (Obj.magic t))
           (Printexc.to_string (Obj.magic t'))); *)
    Stdlib.compare t t'
  end
module M = Map.Make(O)

module I = Misc.IMap

module OIMap = Map.Make(Oid)
module CbIMap = Map.Make(CbId)

let callbacks_by_oid = ref (OIMap.empty : (callback_id * unit) list M.t OIMap.t)
let callbacks = ref (CbIMap.empty : (object_id * unit ev) CbIMap.t)

let magic_ev : 'a ev -> unit ev = fun ev -> Obj.magic ev
let magic_cb : 'a -> unit = fun cb -> Obj.magic cb

let register_ oid ev cb =
  let id = CbId.gen () in
  let m =
    match OIMap.find_opt oid !callbacks_by_oid with
    | None -> M.empty
    | Some m -> m
  in
  let cbs =
    match M.find_opt ev m with
    | None -> []
           | Some l -> l
  in
  let m = M.add ev ((id, cb) :: cbs) m in
  callbacks_by_oid := OIMap.add oid m !callbacks_by_oid;
  callbacks := CbIMap.add id (oid, ev) !callbacks;
  Log.debug (fun m -> m "Registered callback %s for %s"
     (CbId.to_string id) (Printexc.to_string (Obj.magic ev)));
  id

let register : object_id -> 'a ev -> 'a -> callback_id =
  fun oid ev cb -> register_ oid (magic_ev ev) (magic_cb cb)

let get_ oid ev =
  match OIMap.find_opt oid !callbacks_by_oid with
  | None -> []
  | Some m ->
      match M.find_opt ev m with
      | None -> []
      | Some cbs ->
          List.fold_left
            (fun acc (_id,cb) -> cb :: acc)
            [] cbs

let get : object_id -> 'a ev -> 'a list =
  fun oid ev -> Obj.magic (get_ oid (magic_ev ev))

let unregister =
  let remove id oid ev =
    match OIMap.find_opt oid !callbacks_by_oid with
    | None -> ()
    | Some m ->
        match M.find_opt ev m with
        | None -> ()
        | Some cbs ->
            let cbs = List.filter (fun (id2,_) -> id2 <> id) cbs in
            let m = M.add ev cbs m in
            callbacks_by_oid := OIMap.add oid m !callbacks_by_oid
  in
  fun id ->
    Log.debug (fun m -> m "unregistering callback %s" (CbId.to_string id));
    match CbIMap.find_opt id !callbacks with
    | None -> ()
    | Some (oid, ev) -> remove id oid ev

(*
type handler = ..

type cb = { ev: string; h: handler }

module M = Misc.SMap
module I = Misc.IMap

let mutex = Lwt_mutex.create ()
let callbacks_by_oid = ref I.empty
let callbacks = ref I.empty

let register =
  let cpt = ref 0 in
  fun oid cb ->
    Lwt_mutex.with_lock mutex
      (fun () ->
         incr cpt;
         let id = !cpt in
         let m =
           match I.find_opt oid !callbacks_by_oid with
           | None -> M.empty
           | Some m -> m
         in
         let ids =
           match M.find_opt cb.ev m with
           | None -> []
           | Some l -> id :: l
         in
         let m = M.add cb.ev ids m in
         callbacks_by_oid := I.add oid m !callbacks_by_oid;
         callbacks := I.add id cb.h !callbacks;
         Lwt.return id
      )

let get oid ev_name =
  match I.find_opt oid
  *)
    (*

oid -> cb -> id list

register: oid -> cb -> id
*)
