(** Handling current selection.

Current selection is retrieved using {!val-get}. For widgets
handling user selection, when the user selects something,
the widget will {!val-set} the function called to retrieve the selection.
{!val-unset} removes the selection function.
*)

(**/**)
let selection = ref (None: (unit -> string option) option)
(**/**)

let unset () = selection := None
let set f =
  selection := Some f;
  Log.debug (fun m -> m "Selection set")

let get () =
  match !selection with
  | None -> None
  | Some f ->
      try f ()
      with e ->
          Log.err (fun m -> m "Selection.get: %s" (Printexc.to_string e));
          unset ();
          None