(** Theming widgets.

Theming is done by defining default properties for {!Widget.class-widget}
classes (the [class_] parameter in widget creation) and names
(the [name] parameter in widget creation).

*)

type t (** A theme *)

(** Get variables of theme. *)
val variables : t -> Yojson.Safe.t Misc.SMap.t

(** [get_or_create name] returns the theme with the given [name] if it
  exists, else creates it. *)
val get_or_create : string -> t

(** [set_curent t] sets theme [t] as current theme. It will be used
  by newly created widgets. *)
val set_current : t -> unit

(** [current ()] returns the current theme. *)
val current : unit -> t

(** [init ()] initializes default themes. It is called by {!App.val-init}.*)
val init : unit -> unit

(** [of_class ?name class_] returns the properies associated to
  [class_] and optional [name] in current theme. *)
val of_class : ?name:string -> string -> Props.t

(** [load_theme name json] create or modifies theme [t] from the
  given JSON description. *)
val load_theme : Misc.SMap.key -> Yojson.Safe.t -> unit

(** [to_json] returns a JSON representation of the given theme. *)
val to_json : t -> Yojson.Safe.t

(** [to_string t] returns a JSON string representing the theme [t]. *)
val to_string : t -> string

(** [add_class ?inherits class_ props] associates the properties
  [props] to the given [class_] in the current theme. [inherits] specifies
  which classes this class inherited properties from, in the given order. *)
val add_class :  ?inherits:string list -> string -> Props.t -> unit

(** [extend_current_from_json json] extends current theme with JSON description. *)
val extend_current_from_json : Yojson.Safe.t -> unit
