open Stk
open Stk.Misc

let () = Logs.Src.set_level Log.log_src (Some Logs.Info)
let () = Logs.set_reporter (Misc.lwt_reporter ())

let _ = Tsdl.Sdl.(init Init.(video+events))
let () = Lwt_main.run(App.init())
let canvas_width = 800
let words = 10_000

let random_int32 () =
  let n = Random.int32 Int32.max_int in
  let n = Int32.(add (shift_left n 8) (of_int 0xff)) in
  Log.debug (fun m -> m "random_int32 () => %lX" n);
  n

let fill_canvas ~freeze c dir =
(*  let%lwt words =
    let%lwt ic = Lwt_io.(open_file ~mode:Input "/usr/share/dict/french") in
    let rec iter acc n =
      if n >= words then
        Lwt.return acc
      else
        match%lwt Lwt_io.read_line_opt ic with
        | None -> Lwt.return acc
        | Some word ->
            let word = word ^ " " in
            iter (word :: acc) (n+1)
    in
    let%lwt l = iter [] 0 in
    let%lwt () = Lwt_io.close ic in
    Lwt.return l
  in
*)
  if freeze then c#freeze ;
  let rec iter_stream group y dir str =
    (
      Log.info (fun m -> m "iter_stream group=%a, y=%d, dir=%S"
        Stk.G.pp group#geometry y dir);
     match%lwt Lwt_stream.get str with
     | exception _ -> Lwt.return_unit
     | None -> Lwt.return_unit
     | Some "."
     | Some ".." -> iter_stream group y dir str
     | Some entry ->
         let%lwt () = iter group y (Filename.concat dir entry) in
         iter_stream group (group#geometry.h) dir str
    )
  and iter group y dir =
    Log.info (fun m -> m "iter y=%d, dir=%S" y dir);
    let t = Canvas.label ~group ~y dir in
    let (w,h) = t#text_size in
    Log.info (fun m -> m "text_size=(%d,%d)" w h);
    let _r = Canvas.rect ~group ~y ~w ~h () in
    let _id = _r#connect Widget.Clicked
      (fun _ -> Log.info (fun m -> m "Click on %s" dir); true)
    in
    match%lwt Lwt_unix.stat dir with
    | exception e ->
        Log.err (fun m -> m "%s" (Printexc.to_string e));
        Lwt.return_unit
    | st when st.Unix.st_kind = Unix.S_DIR ->
        let group = Canvas.group ~group ~x:5 ~y:(y+h) () in
        let stream =
          try Lwt_unix.files_of_directory dir
          with e ->
              Log.err (fun m -> m "%s" (Printexc.to_string e));
              Lwt_stream.of_list []
        in
        let%lwt () = iter_stream group 0 dir stream in
        Lwt.return_unit
    | _ ->
        Lwt.return_unit
  in
  let%lwt () =
    Log.info (fun m -> m "c#root props=%a" Stk.Props.pp c#root#props);
    try let%lwt _g = iter c#root 0 dir in Lwt.return_unit
    with e ->
        Log.err (fun m -> m "%s" (Printexc.to_string e));
        Lwt.return_unit
  in
  Lwt.return (if freeze then c#unfreeze)

let fill_widgets canvas =
  let g1 = Canvas.group ~x:100 ~y:200 ~group:canvas#root () in
  let g2 = Canvas.group ~x:200 ~y:200 ~group:canvas#root () in
  let p = Props.create () in
  Props.(
   set p bg_color Color.yellow;
   set p fg_color Color.darkblue;
   set p fill true ;
   set p border_width (trbl__ 10) ;
   set p border_color Color.(trbl_ red green blue pink) ;
  );
  let bg1 = Canvas.rect ~props:p ~x: 80 ~w:100 ~h:50 ~group:g1 () in
  let bg2 = Canvas.rect ~props:p ~y:150 ~w:100 ~h:50 ~group:g2 () in

  let b_props =
    let p = Props.create () in
    Props.(set p bg_color (Int32.of_int 0xff000066) ;
      set p Pack.inter_padding 40);
    p
  in
  let b1 = Canvas.vbox ~props:b_props ~group:g1 () in
  let b2 = Canvas.hbox ~props:b_props ~group:g2 () in
  let words = [ "hello"; "the" ; "world" ; "!"] in
  let f (b:unit Canvas.box) w =
    let l = Text.label ~pack:b#pack ~text:w () in
    let fd = l#get_p Props.font_desc in
    l#set_p Props.font_desc
      { fd with size = 14; bold = true };
    l#set_p Props.fg_color Color.darkgreen
  in
  List.iter (f b1) words ;
  List.iter (f b2) words ;
  Lwt.return_unit

let mk_win () =
  let (win,scr) = App.create_scrolled_window
    ~resizable:true
    ~w: 400 ~h: 600 "stk canvas" in
  let hbox = Stk.Pack.hbox ~pack:scr#set_child () in
  let canvas_dirs = Canvas.canvas ~pack:hbox#pack () in
  let canvas_widgets = Canvas.canvas ~pack:hbox#pack () in
  (win, canvas_dirs, canvas_widgets)

let main () =
  let root =
    if Array.length Sys.argv > 1 then
      Sys.argv.(1)
    else
      Filename.current_dir_name
  in
  let (_win, canvas_dirs, canvas_widgets) = mk_win () in
  Lwt.join [Stk.App.run () ;
    fill_canvas ~freeze:true canvas_dirs root ;
    fill_widgets canvas_widgets ;
  ]

let () =
  Lwt_main.run (main ())