open Stk
open Stk.Misc

(*let () = Logs.Src.set_level Log.log_src (Some Logs.Debug)*)
let () = Logs.set_reporter (Misc.lwt_reporter ())

let _ = Tsdl.Sdl.(init Init.(video+events))
let () = Lwt_main.run(App.init())

let gen_words file nb_words =
  let%lwt ic = Lwt_io.(open_file ~mode:Input file) in
  let rec iter acc n =
    if n >= nb_words then
      Lwt.return acc
    else
      match%lwt Lwt_io.read_line_opt ic with
      | None -> Lwt.return acc
      | Some word -> iter (word :: acc) (n+1)
  in
  let%lwt l = iter [] 0 in
  let%lwt () = Lwt_io.close ic in
  Lwt.return (List.rev l)

let mk_win () =
  let win = App.create_window
    ~flags:Tsdl.Sdl.Window.resizable
    ~w: 400 ~h: 600 "stk clist"
  in
  let box = Pack.hbox ~pack:win#set_child () in
  let fixed = Bin.fixed_size ~w:50 ~pack:box#pack () in
  let scr = Bin.scrollbox ~pack:box#pack () in
  let clist = Clist.clist
    (*~selection_mode:Props.Sel_browse*)
    ~show_headers:true ~pack:scr#set_child ()
  in
  let col_word =
    let props = Props.empty () in
    Props.(set props halign 0.0);
    let mk = Clist.string_cell ~props (fun s -> (s, None)) in
    let sort_fun = String.compare in
    Clist.column ~sort_fun ~title:"Word" mk
  in
  ignore(clist#add_column col_word);

  let _ = clist#connect_row_inserted
    (fun row str -> Log.app (fun m -> m "Row inserted: %d, %S" row str))
  in
  let _ = clist#connect_row_removed
    (fun row str -> Log.app (fun m -> m "Row inserted: %d, %S" row str))
  in
  let _ = clist#connect_data_set
    (fun data -> Log.app (fun m -> m "Data set: %d elements" (List.length data)))
  in
  let _ = clist#connect_row_selected
    (fun row str -> Log.app (fun m -> m "Row selected: %d, %S" row str))
  in
  let _ = clist#connect_row_unselected
    (fun row str -> Log.app (fun m -> m "Row unselected: %d, %S" row str))
  in
  let _ = clist#connect_row_updated
    (fun row ~prev ~now -> Log.app (fun m -> m "Row updated: %d, %S => %S" row prev now))
  in
  win, clist

let word_file = ref "/usr/share/dict/french"

let main () =
  if Array.length Sys.argv > 1 then word_file := Sys.argv.(1) ;
  let (_win, clist) = mk_win () in
  let insert_words () =
    let%lwt words = gen_words !word_file 50 in
    clist#freeze ;
    List.iter (fun w ->
       Log.warn (fun m -> m "inserting %S" w);
       ignore(clist#insert w)) words;
    clist#unfreeze;
    let col_length =
      let props_even = Props.empty () in
      let props_odd = Props.empty () in
      Props.(set props_even bg_color Color.pink);
      Props.(set props_odd bg_color Color.cyan) ;
      let mk =
        let props = Props.empty () in
        Props.(set props halign 0.0);
        Props.(set props fill true);
        let of_data s =
          let len = Utf8.length s in
          (len, Some (if len mod 2 = 0 then props_even else props_odd))
        in
        Clist.int_cell ~props of_data
      in
      let sort_fun s1 s2 =
        Int.compare (Utf8.length s1) (Utf8.length s2)
      in
      Clist.column ~sort_fun ~title:"Length" mk
    in
    let n = 10_000 in
    Log.app (fun m -> m "inserting %d words" n);
    let%lwt words = gen_words !word_file n in
    clist#set_list words;
    Log.app (fun m -> m "%d words added" n);
    ignore(clist#add_column col_length);


    let%lwt () = Lwt_unix.sleep 3.0 in
    clist#remove_column col_length;
    Log.app (fun m -> m "inserting %d words" n);
    let%lwt words = gen_words !word_file n in
    clist#set_list words;

    let col_height =
      let mk d =
        let props = Props.empty () in
        let fd = Props.(get props font_desc) in
        let size = 10 + Random.int 10 in
        Props.(set props font_desc { fd with Font.size} );
        let of_data s = (size, None) in
        Clist.int_cell ~props of_data d
      in
      Clist.column ~title:"Random height" mk
    in
    ignore(clist#add_column col_height);

    let%lwt () = Lwt_unix.sleep 3.0 in
    Log.warn (fun m -> m "setting row_height to 12");
    clist#set_row_height (12);

    let%lwt () = Lwt_unix.sleep 3.0 in
    Log.warn (fun m -> m "setting row_height to -1");
    clist#set_row_height (-1);

    let%lwt () = Lwt_unix.sleep 3.0 in
    Log.warn (fun m -> m "Capitalizing words");
    clist#freeze ;
    Array.iteri
      (fun i word -> clist#set_row i (String.capitalize_ascii word))
      clist#data ;
    clist#unfreeze ;

    Lwt.return_unit
  in
  Lwt.join [Stk.App.run (); insert_words ()]

let () = Lwt_main.run (main ())