open Stk
open Stk.Misc

let () = Logs.set_reporter (Misc.lwt_reporter ())

let _ = Tsdl.Sdl.(init Init.(video+events))
let () = Lwt_main.run (App.init ())

let mk_win () =
  let win = App.create_window ~show:false
    (*~flags:Tsdl.Sdl.Window.resizable*) "stk dialogs" in
  let box = Pack.vbox ~pack:win#set_child () in
  Props.(box#set_p padding (trbl 2 4 2 4)) ;
  let hbox1 = Pack.hbox ~pack:(box#pack ~vexpand:0) () in
  let (b1,_) = Button.text_button ~text:"click to set text"
    ~pack:(hbox1#pack ~hexpand:0) () in
  let label1 = Text.label ~pack:hbox1#pack () in

  let _id = b1#connect Stk.Widget.Activated
    (fun _ ->
       let dialog = Dialog.dialog ~behaviour:(`Modal_for win) "Setting text" in
       let entry = Edit.entry ~pack:dialog#content_area#set_child ~text:label1#text () in
       let _ = dialog#add_text_button
         ~return:(fun () -> Some (`Ok (entry#text ()))) "Ok"
       in
       let _ = dialog#add_text_button
         ~return:(fun () -> Some (`Ok_upper (entry#text()))) "ascii-uppercased"
       in
       let _ = dialog#add_text_button
         ~return:(fun () -> Some (`Ok_lower (entry#text()))) "ascii-lowercased"
       in
       let _ = dialog#add_text_button
         ~return:(fun () -> None) "Cancel"
       in
       dialog#run (fun res ->
          let () =
            match res with
            | None -> Stk.Log.app (fun m -> m "dialog#run returned None")
            | Some (`Ok text) -> label1#set_text text
            | Some (`Ok_upper text) -> label1#set_text (String.uppercase_ascii text)
            | Some (`Ok_lower text) -> label1#set_text (String.lowercase_ascii text)
          in
          Lwt.return_unit
       ))
  in

  let hbox2 = Pack.hbox ~pack:(box#pack ~vexpand:0) () in
  let (b2,_) = Button.text_button ~text:"click to set text"
    ~pack:(hbox2#pack ~hexpand:0) () in
  let label2 = Text.label ~pack:hbox2#pack () in
  let (simple_input, _get_text) = Dialog.simple_input
    ~behaviour:`Hide_on_return ~msg:"Text:" "Simple input"
  in
  let _ = b2#connect Stk.Widget.Activated
    (fun () ->
       simple_input#run
         (function
          | None -> Lwt.return_unit
          | Some txt -> label2#set_text txt; Lwt.return_unit)
    )
  in
  let _ = win#connect Widget.Destroy (fun _ -> App.quit (); true) in
  win#show;
  win

let main () =
  let _win = mk_win () in
  Lwt.join [Stk.App.run ()]

let () =
  Lwt_main.run (main ())