open Stk
open Stk.Misc

(*let () = Logs.Src.set_level Log.log_src (Some Logs.Debug)*)

let () = Logs.set_reporter (Misc.lwt_reporter ())

let _ = Tsdl.Sdl.(init Init.(video+events))
let () = Lwt_main.run(Stk.App.init())

let () = Props.(set default font_desc
  (let f = get default font_desc in
   {f  with size = 16 })
  )
let mk_win () =
  let (win,scr) = App.create_scrolled_window
    ~flags:Tsdl.Sdl.Window.resizable
    ~w: 400 ~h: 600 "stk test_entry" in
  let vbox = Pack.vbox ~pack:scr#set_child () in

  let hbox1 = Pack.hbox ~pack:(vbox#pack ~vexpand:0) () in
  let l1 = Text.label ~text: "first entry:"
    ~pack:(hbox1#pack ~hexpand:0) ()
  in
  let e1 = Edit.entry ~text:"hello"
    ~pack:(hbox1#pack ~hfill:true ~hexpand:1) () in

  let hbox2 = Pack.hbox ~pack:(vbox#pack ~vexpand:0) () in
  let l2 = Text.label ~text: "second entry:"
    ~pack:(hbox2#pack ~hexpand:0) ()
  in
  let e2 = Edit.entry ~text:"world"
    ~pack:(hbox2#pack ~hfill:true ~hexpand:1) ()
  in

  let hbox3 = Pack.hbox ~pack:(vbox#pack ~vexpand:0) () in
  let l3 = Text.label ~text: "third entry:"
    ~pack:(hbox3#pack ~hexpand:0) ()
  in
  let e3 = Edit.entry
    ~pack:(hbox3#pack ~hfill:true ~hexpand:1) () in

  let canvas = Canvas.canvas ~pack:vbox#pack () in
  let () = canvas#set_p Props.can_focus true in
  let class_ = "mygroup" in
  let () = Theme.add_class class_
    Props.(
      let p = empty () in
      set p border_width (trbl__ 2) ;
      set p border_color Color.(trbl_ red green blue yellow) ;
      set p bg_color Color.pink;
      set p fill true;
      p)
  in
  let g1 = Canvas.group ~class_ ~group:canvas#root
    ~x: 50 ~y:50 () in
  let t1 = Canvas.label ~group:g1 (e1#text()) in

  let g2 = Canvas.group ~class_ ~group:canvas#root
    ~x:100 ~y:100 () in
  let fh2 = Canvas.fixed_size ~group:g2 ~h:50 () in
  let t2 = Text.label ~valign:0.333 ~pack:fh2#set_child ~text:(e2#text ()) () in

  let g3 = Canvas.group ~class_ ~group:canvas#root
    ~x: 200 ~y:50 () in
  let t3 = Canvas.label ~group:g3 (e3#text ()) in

  let g4 = Canvas.group ~class_ ~group:canvas#root ~x: 200 ~y: 200 () in
  let box = Canvas.hbox ~group:g4 () in
  let e4 = Edit.entry ~pack:box#pack ~text:"entry in canvas" () in
  Log.info (fun m -> m "canvas can focus: %b" (canvas#get_p Props.can_focus));

  let f (e:Widget.widget) t =
    let _ = e#connect (Object.Prop_changed Props.is_focus)
      (fun ~prev ~now ->
         Log.info (fun m -> m "%s is_focus: %b" e#me now))
    in
    let _ = e#connect (Object.Prop_changed Props.has_focus)
      (fun ~prev ~now ->
         Log.info (fun m -> m "%s has_focus: %b" e#me now))
    in
    let _ = e#connect (Object.Prop_changed Props.text)
      (fun ~prev ~now -> t#set_text now)
    in
    ()
  in
  let () = f e1#coerce t1 in
  let () = f e2#coerce t2 in
  let () = f e3#coerce t3 in

  let _ = win#connect (Object.Prop_changed Props.has_focus)
    (fun ~prev ~now ->
      Log.info (fun m ->
          if now then
            m "window gained input focus !"
          else
            m "window lost input focus !"
       )
    )
  in
  win

let main () =
  let _win = mk_win () in
  Lwt.join [Stk.App.run ()]

let () =
  Lwt_main.run (main ())