open Stk
open Stk.Misc

let () = Logs.set_reporter (Misc.lwt_reporter ())

let () = Random.self_init ()

type Widget.wdata +=
| Items_align of Flex.item_alignment
| Justification of Flex.justification

let main () =
  let%lwt () = App.init () in
  let win = App.create_window ~flags:Tsdl.Sdl.Window.resizable
    ~w:400 ~h:600 "test flex"
  in
  let box = Pack.vbox ~pack:win#set_child () in
  let (b_orient,l) = Button.text_togglebutton ~text:"Flex orientation: horizontal"
    ~pack:(box#pack ~vexpand:0) ()
  in
  let hbox_just = Pack.hbox ~pack:(box#pack ~vexpand:0) () in
  let _ = Text.label ~text:"Justification:" ~pack:(hbox_just#pack ~vexpand:0 ~hexpand:0) () in
  let group_just = Button.group () in
  List.iter
    (fun a ->
      let text = Flex.string_of_justification a in
      let (b,_) = Stk.Button.text_radiobutton
         ~group:group_just ~text ~pack:(hbox_just#pack ~hexpand:0) ()
       in
       b#set_margin (Props.trbl ~top:0 ~left:5 ~bottom:0 ~right:5);
       b#set_wdata (Some (Justification a))
    )
   [`Center ; `End ; `Justified ; `Space_around ; `Space_between ; `Start];

  let hbox_align = Pack.hbox ~pack:(box#pack ~vexpand:0) () in
  let _ = Text.label ~text:"Items alignment:" ~pack:(hbox_align#pack ~vexpand:0 ~hexpand:0) () in
  let group_align = Button.group () in
  List.iter
    (fun a ->
      let text = Flex.string_of_item_alignment a in
      let (b,_) = Stk.Button.text_radiobutton
         ~group:group_align ~text ~pack:(hbox_align#pack ~hexpand:0) ()
       in
       b#set_margin (Props.trbl ~top:0 ~left:5 ~bottom:0 ~right:5);
       b#set_wdata (Some (Items_align a))
    )
    [`Baseline ; `Center ; `End ; `Start ; `Stretch];
  let scr = Bin.scrollbox ~pack:box#pack () in
  let flex = Flex.flex
    ~justification:`Start
    ~items_alignment:`Baseline
    ~inter_space:0
    ~collapse_spaces:true
    ~wrap_on_break:true
    ~pack:scr#set_child ()
  in
  flex#set_bg_color Color.white;
  flex#set_fill true;
  let _ = group_just#connect (Object.Prop_changed Button.active_widget)
    (fun ~prev ~now ->
      match now#wdata with
      | Some (Justification a) -> flex#set_justification a
      | _ -> ())
  in
  let _ = group_align#connect (Object.Prop_changed Button.active_widget)
    (fun ~prev ~now ->
      match now#wdata with
      | Some (Items_align a) -> flex#set_items_alignment a
      | _ -> ())
  in
  let insert_text (f:unit Flex.flex) ?(a=Props.Baseline) text size  =
    let (margin, kind) =
      match text with
      | " " -> 0, Some (`Space true)
      | "\n" -> 0, Some `Break
      | _ -> 0, None
    in
    let t = Text.label ~text ~pack:(f#pack ?kind) () in
    t#set_padding (Props.trbl__ 0);
    ignore(margin);
    t#set_margin (Props.trbl ~top:0 ~right:margin ~bottom:0 ~left:margin);
    t#set_border_width (Props.trbl__ 1);
    t#set_border_color (Props.trbl__ Color.red);
    let fd = t#font_desc in
    let fd = { fd with Font.size } in
    t#set_font_desc fd ;
    t#set_text_valign a;
    ()
  in
  ignore(flex#pack_space ());
  insert_text flex "bla" 14;
  ignore(flex#pack_space ());
  insert_text flex "ble" 23;
  insert_text flex "bli" 23;
  ignore(flex#pack_space ());
  insert_text flex "bloooooo" 30;
  ignore(flex#pack_space ());
  insert_text flex "blu" 15;
  ignore(flex#pack_space ());
  insert_text flex "bli" 25;
  ignore(flex#pack_space ());

  let block = Text.label ~text:"block" ~pack:(flex#pack ~kind:`Block) () in
  block#set_bg_color Color.red;
  block#set_fill true;
  block#set_margin (Props.trbl__ 5);
  block#set_border_color (Props.trbl__ Color.brown);
  block#set_border_width (Props.trbl__ 2);

  ignore(flex#pack_space ());

  let block = Flex.flex
    ~wrap_on_break:false
    ~justification:`Start
    ~items_alignment:`Baseline
      ~pack:(flex#pack ~kind:`Block) ()
  in
  block#set_p Flex.expand 1;
  block#set_bg_color Color.yellow;
  block#set_fill true;
  block#set_margin (Props.trbl__ 5);
(*  block#set_border_color (Props.trbl__ Color.brown);
  block#set_border_width (Props.trbl__ 2);*)
  insert_text block "coucou" 25;
  ignore(block#pack_space());
  insert_text block "le" 22;
  ignore(block#pack_break());
  insert_text block "block" 32;
  ignore(block#pack_space());

  ignore(flex#pack_space ());

  let flex2 = Flex.flex ~name:"flex2" ~wrap:false ~wrap_on_break:true
    ~justification:`Center ~pack:(flex#pack) () in
  flex2#set_p Flex.expand 1;

  insert_text flex2 "fla" 34;
  ignore(flex2#pack_space ());
  insert_text flex2 "fle" 23;
  insert_text flex2 "fli" 23;
  insert_text flex2 "\n" 15;
  insert_text flex2 "flo" 30;
  insert_text flex2 "flu" 15;
  insert_text flex2 "fli" 12;

  ignore(flex#pack_space ());

(*  insert_text flex "\n" 14;
  insert_text flex "\n" 14;*)
  insert_text flex "gle" 23;
  insert_text flex ~a:Props.Super "2" 18;
  ignore(flex#pack_space ());
  ignore(flex#pack_space ());
  ignore(flex#pack_space ());
  ignore(flex#pack_space ());
  insert_text flex ~a:Props.Top "Top" 18;
  ignore(flex#pack_space ());
  insert_text flex ~a:Props.Bottom "Bottom" 18;
  ignore(flex#pack_space ());
  insert_text flex "gli" 23;
  insert_text flex ~a:Props.Sub "i" 18;
  ignore(flex#pack_space ());
  let (b,_) = Button.text_button ~text:"button" ~pack:flex#pack() in
  insert_text flex "glo" 30;
  ignore(flex#pack_space ());
  insert_text flex ~a:Props.Text_top "texttop" 15;
  ignore(flex#pack_space ());
  let (b,_) = Button.text_button ~text:"button" ~pack:flex#pack() in
  insert_text flex "glu" 15;
  ignore(flex#pack_space ());
  insert_text flex ~a:Props.Text_bottom "textbottom" 15;
  ignore(flex#pack_space ());
  insert_text flex "gli" 12;

  ignore(b_orient#connect (Object.Prop_changed Button.active)
    (fun ~prev ~now ->
       let (o, str) = if now
         then (Props.Vertical, "vertical")
         else (Props.Horizontal, "horizontal")
       in
       flex#set_orientation o;
       l#set_text (Printf.sprintf "Flex orientation: %s" str)));

  Log.warn (fun m -> m "%a" Widget.pp_widget_tree win#wtree);
  Lwt.join [Stk.App.run ()]

let () = Lwt_main.run (main ())
