open Stk
open Stk.Misc

let () = Logs.Src.set_level Log.log_src (Some Logs.Info)

let () = Logs.set_reporter (Misc.lwt_reporter ())

let _ = Tsdl.Sdl.(init Init.(video+events))
let () = Lwt_main.run(Stk.App.init())

let mk_win () =
  let (win,scr) = App.create_scrolled_window
    ~flags:Tsdl.Sdl.Window.resizable
    ~w: 400 ~h: 600 "stk canvas" in
  let box = Pack.vbox ~pack:scr#set_child () in
  let (button1, label1) = Button.text_button
    ~text:"button 1" ~pack:(box#pack ~hfill:true ~vexpand:0) ()
  in
  let (button2, label2) = Button.text_button
    ~text:"button 2" ~pack:(box#pack ~hfill:true ~vexpand:0) ()
  in
  let f (b:Widget.widget) =
    let _ = b#connect (Object.Prop_changed Props.is_focus)
      (fun ~prev ~now ->
         Log.info (fun m -> m "%s is_focus: %b" b#me now))
    in
    let _ = b#connect (Object.Prop_changed Props.has_focus)
      (fun ~prev ~now ->
         Log.info (fun m -> m "%s has_focus: %b" b#me now))
    in
    ()
  in
  let () = f button1#coerce in
  let () = f button2#coerce in

  let _ = win#connect (Object.Prop_changed Props.has_focus)
    (fun ~prev ~now ->
      Log.info (fun m ->
          if now then
            m "window gained input focus !"
          else
            m "window lost input focus !"
       )
    )
  in
  win

let main () =
  let _win = mk_win () in
  Lwt.join [Stk.App.run ()]

let () =
  Lwt_main.run (main ())