open Stk
open Stk.Misc

(*let () = Logs.Src.set_level Log.log_src (Some Logs.Debug)*)
let () = Logs.set_reporter (Misc.lwt_reporter ())

let _ = Tsdl.Sdl.(init Init.(video+events))
let () = Lwt_main.run (App.init ())

let canvas_width = 800

let fill_canvas c =
  c#freeze ;
  let colors = Array.of_list Stk.Color.named_colors in
  let nb_colors = Array.length colors in
  let fonts = Stk.Font.fonts () in
  let fonts = List.sort (fun f1 f2 ->
    String.compare f1.Stk.Font.family f2.family)
    fonts
  in
  let f y desc =
    let fn = Font.get desc in
    let is_fw = Stk.Font.((font_metrics fn).font_is_fixed_width) in
    let text desc = Printf.sprintf "%s (fixed width: %d) 😀"
      (Stk.Font.string_of_font_desc desc) (is_fw)
    in
    let t =
      let desc = { desc with Stk.Font.size = 30 ; outline=1} in
      let props = Props.create () in
      Props.(set props font_desc desc) ;
      Props.(set props fg_color colors.(Random.int nb_colors)) ;
      Canvas.label ~props ~group:c#root ~x:0 ~y (text desc)
    in
    let (_,h) = t#text_size in
    let y = y + h + 2 in
    let t =
      let desc = { desc with Stk.Font.size = 14 } in
      let props = Props.create () in
      Props.(set props font_desc desc) ;
      Canvas.label ~props ~group:c#root ~x:0 ~y (text desc)
    in
    let (_,h) = t#text_size in
    (y + h + 2)
  in
  let _ = List.fold_left f 0 fonts in
  c#unfreeze

let mk_win () =
  let (win,scr) = App.create_scrolled_window
    ~resizable:true ~w: 400 ~h: 600 "stk canvas" in
  let canvas = Canvas.canvas ~pack:scr#set_child () in
  (win, canvas)

let main () =
  Lwt_main.run (
   let dirs = Array.to_list (Array.sub Sys.argv 1 (Array.length Sys.argv - 1)) in
   let dirs = List.map (fun s -> (s, true)) dirs in
   Font.load_fonts ~dirs ());
  let (_win, canvas) = mk_win () in
  fill_canvas canvas;
  Lwt.join [Stk.App.run ()]

let () = Lwt_main.run (main ())