open Stk
open Stk.Misc
open Tsdl

let () = Logs.set_reporter (Misc.lwt_reporter ())

let _ = Tsdl.Sdl.(init Init.(video+events))
let () = Lwt_main.run(Stk.App.init())

let () = Props.(set default font_desc
  (let f = get default font_desc in
   {f  with size = 16 })
  )
let mk_win () =
  let (win,scr) = App.create_scrolled_window
    ~flags:Tsdl.Sdl.Window.resizable
    ~w: 400 ~h: 600 "stk test_keys" in
  let vbox = Pack.vbox ~pack:scr#set_child () in

  let hbox0 = Pack.hbox ~pack:(vbox#pack ~vexpand:0) () in
  let _ = Text.label ~text:"Key pressed:"
    ~pack:(hbox0#pack ~hexpand:0) ()
  in
  let l0 = Text.label ~pack:hbox0#pack ()in

  let hbox1 = Pack.hbox ~pack:(vbox#pack ~vexpand:0) () in
  let l1 = Text.label ~text: "Print hello:"
    ~pack:(hbox1#pack ~hexpand:0) ()
  in
  let k1 = ref (Key.keystate ~mods:Sdl.Kmod.ctrl Sdl.K.h) in
  let e1 = Edit.entry ~text:(Key.string_of_keystate !k1)
    ~pack:(hbox1#pack ~hfill:true ~hexpand:1) () in

  let hbox2 = Pack.hbox ~pack:(vbox#pack ~vexpand:0) () in
  let l2 = Text.label ~text: "Print world:"
    ~pack:(hbox2#pack ~hexpand:0) ()
  in
  let k2 = ref (Key.keystate ~mods:Sdl.Kmod.(alt+shift) Sdl.K.w) in
  let e2 = Edit.entry ~text:(Key.string_of_keystate !k2)
    ~pack:(hbox2#pack ~hfill:true ~hexpand:1) ()
  in
  let err_label = Text.label ~pack:(vbox#pack ~vexpand:0) () in
  err_label#set_p Props.fg_color Color.red ;
  err_label#set_visible false ;
  let label = Text.label ~pack:(vbox#pack ~vexpand: 10) () in
  let on_key_press kev =
    l0#set_text (Sdl.get_key_name kev.Widget.key);
    if Key.match_keys !k1 kev.Widget.key kev.Widget.mods then
      (label#set_text "hello" ; true)
    else if Key.match_keys !k2 kev.Widget.key kev.Widget.mods then
        ( label#set_text "world" ; true)
      else
        false
  in
  let f (e:Edit.entry) k =
    let cb ~prev ~now =
      try
        k := Key.keystate_of_string now;
        err_label#set_visible false
      with e ->
        err_label#set_text (Printexc.to_string e) ;
        err_label#set_visible true
    in
    e#connect (Object.Prop_changed Props.text) cb
  in
  let _ = f e1 k1 in
  let _ = f e2 k2 in
  let _id = win#connect Widget.Key_pressed on_key_press in
  win

let main () =
  let _win = mk_win () in
  Lwt.join [Stk.App.run ()]

let () = Lwt_main.run (main ())
