open Stk
open Stk.Misc

let () = Logs.set_reporter (Misc.lwt_reporter ())

(*let () = Logs.set_level ~all:true (Some Logs.Debug)*)

let text = "Lorem ipsum dolor sit amet, consectetur adipiscing elit.
Proin vel nisi a enim porttitor dignissim. Sed lacinia orci
eros, sed viverra metus venenatis sit amet. Nulla sed
quam quam. Donec nec justo maximus, consequat orci eu, lobortis ex.
Proin interdum porttitor lectus, vel scelerisque nunc vehicula
cursus. Ut blandit vehicula diam. In a ultricies
metus. Aenean eget libero commodo nulla placerat sollicitudin
eleifend id dui. Aliquam erat volutpat. Aliquam erat
volutpat. Proin bibendum aliquam facilisis. Suspendisse
potenti. Vivamus vel viverra nunc. Maecenas aliquam egestas
magna, vel pellentesque ipsum congue id. Sed in leo sit
amet turpis dictum ultrices. Nullam varius nisi augue,
sit amet efficitur massa consectetur a."

let main () =
  let%lwt () = App.init () in
  let win = App.create_window ~flags:Tsdl.Sdl.Window.resizable ~w:400 ~h:600 "test paned" in
  let nb = Notebook.hnotebook ~pack:win#set_child () in
  Wkey.add nb#coerce (Key.keystate ~mods:Tsdl.Sdl.Kmod.ctrl Tsdl.Sdl.K.k)
    (fun () ->
       match nb#active_page with
       | None -> ()
       | Some n -> ignore(nb#remove_page n)
    );
  let make_tv label =
    let label = Text.label ~text:label () in
    let scr = Bin.scrollbox ~pack:(nb#pack (*~pos:0*) ~label:label#as_widget) () in
    let tv = Textview.(textview ~wrap_mode:Wrap_char ~pack:scr#set_child ()) in
    tv#set_source_language (Some "ocaml") ;
    tv#insert text ;
    Textview.set_default_key_bindings
      ~display_state: (fun ~after_handler state ->
         Log.app (fun m -> m "State: (after_handler:%b) %a"
            after_handler Wkey.pp_keyhit_state state))
      tv;
    tv
  in
  let%lwt () =
    let len = Array.length Sys.argv in
    if len <= 1 then
      let _tv1 = make_tv "text1" in
      let tv2 = make_tv "text2" in
      tv2#set_theme "cobalt" ;
      let _tv3 = make_tv "text3" in
      let _tv4 = make_tv "text4" in
      let _tv5 = make_tv "text5" in
      Lwt.return_unit
    else
      Lwt_list.iter_p
        (fun file ->
           let tv = make_tv (Filename.basename file) in
           let _ = tv#delete () in
           let%lwt s = Lwt_io.(with_file ~mode:Input file read) in
           tv#insert s;
           Lwt.return_unit
        )
        (Array.to_list (Array.sub Sys.argv 1 (len - 1)))
  in
  let%lwt () = Lwt.join [Stk.App.run ()] in
  Lwt.return_unit

let () = Lwt_main.run (main ())
