open Stk
open Stk.Misc

let () = Logs.set_reporter (Misc.lwt_reporter ())

let () = Random.self_init ()
let main () =
  let%lwt () = App.init () in
  let (win,scr) = App.create_scrolled_window ~flags:Tsdl.Sdl.Window.resizable
    ~w:500 ~h:600 "test pack"
  in
  let mk_int_param (box:unit Pack.box) label prop =
    let hbox = Pack.hbox ~pack:(box#pack ~vexpand:0) () in
    let wl = Text.label ~text:(label^":") ~valign:0. ~pack:(hbox#pack ~hexpand:0) () in
    let e = Edit.entry ~pack:(hbox#pack ~vfill:false) () in
    e#set_text (string_of_int (box#get_p prop));
    let _ = e#connect (Object.Prop_changed Props.text)
      (fun ~prev ~now ->
         try
           let n = int_of_string (e#text ()) in
           box#set_p prop n
         with _ -> ())
    in
    ()
  in
  let mk_bool_param (box:unit Pack.box) ~ichar ~achar label prop =
    let hbox = Pack.hbox ~pack:(box#pack ~vexpand:0) () in
    let (b,l) = Button.text_checkbutton ~text:label ~pack:(hbox#pack ~hexpand:0) () in
    b#set_active (box#get_p prop);
    b#set_indicator_active_char (Uchar.of_int achar);
    b#set_indicator_inactive_char (Uchar.of_int ichar);
    let _ = b#connect (Object.Prop_changed Button.active)
      (fun ~prev ~now -> box#set_p prop now)
    in
    ()
  in
  let mk_box (box:unit Pack.box) =
    let vbox = Pack.vbox ~pack:box#pack () in
    vbox#set_margin__ (5 + Random.int 15);
    vbox#set_border_width__ 2;
    Props.(vbox#set_p border_color (trbl__ (Color.random())));
    mk_int_param vbox "hexpand" Props.hexpand ;
    mk_int_param vbox "vexpand" Props.vexpand ;
    mk_bool_param vbox ~achar:8660 ~ichar:8596 "hfill" Props.hfill ;
    mk_bool_param vbox ~achar:8661 ~ichar:8597 "vfill" Props.vfill ;
    vbox
  in
  let vbox = Pack.vbox ~pack:scr#set_child () in
  let b1 = mk_box vbox in
  let b2 = mk_box vbox in
  let hbox = Pack.hbox ~pack:vbox#pack () in
  let b3 = mk_box hbox in
  let b4 = mk_box hbox in
  let b5 = mk_box hbox in
  let b6 = mk_box hbox in
  let b7 = mk_box vbox in
  let b8 = mk_box vbox in
  Log.warn (fun m -> m "%a" Widget.pp_widget_tree win#wtree);
  Lwt.join [Stk.App.run ()]

let () = Lwt_main.run (main ())
