open Stk
open Stk.Misc

let () = Logs.set_reporter (Misc.lwt_reporter ())

let nb_texts = 100_000
let mk_texts () =
  let props = Props.create () in
  let rec iter acc n =
    if n >= nb_texts then
      acc
    else
      (
       let t = new Text.textured_text () in
       let fn = Props.get_font props in
       t#set_text fn
         Props.(get props fg_color) (string_of_int n);
       iter (t::acc) (n+1)
      )
  in
  let l = iter [] 0 in
  prerr_endline (Printf.sprintf "%d texts created!" nb_texts);
  l

let main () =
  let%lwt () = App.init () in
  let (_win, scr) = App.create_scrolled_window ~resizable:true
    ~w:400 ~h:600 "test text" in
  let fs = Bin.fixed_size ~w:1000 ~h:100_000
    ~pack:scr#set_child ()
  in
  let texts = mk_texts () in
  let%lwt () = Lwt.join [Stk.App.run ()] in
  List.iter (fun t -> t#destroy_surface) texts;
  Lwt.return_unit

let () = Lwt_main.run (main ())