open Stk
open Stk.Misc

let () = let> () = Tsdl.Sdl.(init Init.(video+events))  in ()
let src = Logs.Src.create "test_textlog"

let stk_log_levels = List.map
  (fun src -> (src, Logs.Src.level src))
    (Stk.Log.src_list ())
let () = Logs.set_level ~all:true (Some Logs.Debug)
let () =
  List.iter (fun (src, level) -> Logs.Src.set_level src level) stk_log_levels

let random_string () = String.init 150
  (fun i -> if i mod 10 = 0 then ' ' else Char.chr (97 + Random.int 26))
let main () =
  let%lwt () = App.init () in
  let win = App.create_window ~resizable:true ~w:600 ~h:600 "test_textlog" in
  let vbox = Pack.vbox ~pack:win#set_child () in
  let hbox_buttons = Pack.hbox ~pack:(vbox#pack ~vexpand:0) () in
  let f (level, k) =
    let (b,t) = Button.text_button ~text:(Logs.level_to_string (Some level))
      ~pack:hbox_buttons#pack ()
    in
    t#set_halign 0.5;
    let on_activate () =
      let str = random_string () in
      Logs.msg ~src level
        (fun m -> m "(%s)%s" (Logs.level_to_string (Some level)) str)
    in
    let _ = b#connect Widget.Activated on_activate in
    Wkey.add win#coerce (Key.keystate k) (fun () -> b#activate)
  in
  List.iter f Logs.[
    Debug, Tsdl.Sdl.K.d ;
    Info, Tsdl.Sdl.K.i ;
    Warning, Tsdl.Sdl.K.w ;
    Error, Tsdl.Sdl.K.e ;
    App, Tsdl.Sdl.K.a] ;
  let scr = Bin.scrollbox ~pack:vbox#pack () in
  let textlog = Textlog.textlog ~pack:scr#set_child () in
  textlog#set_theme "terminal";
  
  Logs.set_reporter (Textlog.reporter textlog) ;
  let rec loop () =
    let%lwt () = Lwt_unix.sleep 2.0 in
    Logs.info ~src (fun m -> m "tic");
    loop ()
  in
  Lwt.async loop;
  Stk.App.run ()



let () = Lwt_main.run (main ())