open Stk
open Stk.Misc

(*let () = Rope.max_leaf_size := 10*)

let () = let> () = Tsdl.Sdl.(init Init.(video+events))  in ()
let () = Logs.set_reporter (Misc.lwt_reporter ())

module B = Textbuffer
module Tag = Texttag.T
module Theme = Texttag.Theme
module L = Texttag.Lang

let () = Logs.set_level ~all:true (Some Logs.Warning)
let () = Logs.Src.set_level Textview.log_src (Some Logs.Warning)
(*let () = Logs.Src.set_level B.src (Some Logs.Debug)*)
(*let () = Logs.set_level ~all:true (Some Logs.Info)*)
(*
let () = Logs.set_level ~all:true (Some Logs.Debug)
let () = Logs.Src.set_level Stk.Texture.src (Some Logs.Warning)
*)

let main () =
  let%lwt () = App.init () in
  let theme = Theme.get_or_create "default" in
  let tag =
    let t = Tag.create "tag1" in
    let props = Props.empty () in
    Props.(set props fg_color Color.yellow;
     set props bg_color Color.green;
     set props fill true;
     set props font_desc (Font.font_desc  ~size:24 "Liberation Sans")
    );
    Theme.set_tag theme t props;
    t
  in
  let tag2 =
    let t = Tag.create "tag2" in
    let props = Props.empty () in
    Props.(set props fg_color Color.red;
     set props bg_color Color.violet;
     set props fill true;
     set props editable false;
     set props font_desc (Font.font_desc  ~size:16 "Liberation Mono")
    );
    Theme.set_tag theme t props ;
    t
  in
  Log.warn (fun m -> m "tag = %a" Texttag.T.pp tag);
  Log.warn (fun m -> m "tag2 = %a" Texttag.T.pp tag2);
  let%lwt buffers =
    if Array.length Sys.argv <= 1 then
      (
       let b = B.create () in
       B.insert b 0"hello bla bla bla\nall of\nthe\nworld\n!";
       Lwt.return ["none", b]
      )
    else
      Lwt_list.map_p
        (fun f ->
           let%lwt str = Lwt_io.(with_file ~mode:Input f read) in
           let b = B.create () in
           B.insert b 0 str ;
           Lwt.return (f, b)
        )
        (Array.to_list (Array.sub Sys.argv 1 (Array.length Sys.argv - 1)))
  in
  Log.warn (fun m -> m "%d buffers created" (List.length buffers));
  let b =
    match buffers with
    | [] -> assert false
    | (_,b) :: _ ->
        B.set_word_char_re_string b "[a-zA-Z0-9]" ;
        b
  in
  B.add_tag b tag ~start:500 ~size:1 ();
  B.check b;
  let win = App.create_window ~resizable:true
    (*~rflags:Tsdl.Sdl.Renderer.software*)
    ~w:600 ~h:600 "test text" in
  let box = Pack.vbox ~pack:win#set_child () in
  let tv1 =
    let scr = Bin.scrollbox ~pack: box#pack () in
    scr#set_p Props.padding (Props.trbl__ 4) ;
    scr#set_p Props.bg_color Color.darkgreen ;
    scr#set_p Props.fill true ;
    let tv1 = Textview.textview ~buffer:b ~pack:scr#set_child () in
    tv1#set_p Textview.wrap_mode Textview.Wrap_char ;
    tv1#set_p Textview.show_line_numbers true ;
    tv1#set_p Textview.show_line_markers true ;
    let _c1 = tv1#add_cursor ~line:0 ~char: 10 () in
    let _c2 = tv1#add_cursor ~line:10 () in
    tv1
  in
  let tv2 =
    let scr = Bin.scrollbox ~pack: box#pack () in
    scr#set_p Props.padding (Props.trbl__ 4) ;
    scr#set_p Props.bg_color Color.orange ;
    scr#set_p Props.fill true ;
    let tv2 = Textview.textview ~buffer:b ~pack:scr#set_child () in
    tv2#set_p Textview.show_line_numbers true ;
    tv2#set_theme "tango" ;
    tv2
  in
  tv2#insert "A\nB\nC\nD\n" ;

  (*  Log.warn (fun m ->
    let module TT = Texttag.Theme in
    List.iter (fun t -> m "%s: %s" t.TT.name
        (Yojson.Safe.pretty_to_string (TT.to_json t)))
      [tv1#theme ; tv2#theme]
  );
*)
  let str = B.to_string ~start:0 ~size:30 b in
  Log.info (fun m -> m "str[0..30]=%S" str) ;
  let delay = 0.1 in
  Lwt.async
    (fun () ->
       let%lwt () = Lwt_unix.sleep delay in
       B.insert b ~tags:[tag;tag2] 30 "Hello again !!!" ;
       Log.warn (fun m -> m "inserted Hello again !!!");
       let%lwt () = Lwt_unix.sleep delay in
       B.remove_tag b tag2 ~start:30 ~size:10 ();
       let%lwt () = Lwt_unix.sleep delay in
       let size = B.size b in
       B.insert b ~tags:[tag]
         (min size 100) "great hello\nworld\n!";
       Log.warn (fun m -> m "inserted great hello world");
       let%lwt () = Lwt_unix.sleep delay in
       let rec iter n =
         if n > 80 then
           ()
         else
           (
            let str = Utf8.string_of_uchar (Uchar.of_int (64 + Random.int 26)) in
            let size = B.size b in
            let pos = min size (50 + Random.int 50) in
            Log.info (fun m -> m "insert %S at %d" str pos) ;
            let () =
              try B.insert b ~tags:[tag2] pos str
              with (Assert_failure _ as e) ->
                  Log.err (fun m -> m "rope = %a" B.pp b);
                  raise e
            in
            (*let%lwt () = Lwt_unix.sleep 0.3 in*)
            iter (n + 1)
           )
       in
       iter 0 ;
       let str = B.to_string ~start:0 ~size:100 b in
       Log.info (fun m -> m "after inserts: str[0..100]=%S" str) ;

       let str = B.delete b ~start:0 ~size:6 in
       Log.info (fun m -> m "deleted %S" str);
       let%lwt () = Lwt_unix.sleep delay in
       let size = B.size b in
       let str = B.delete b ~start: 14 ~size: (min (size-14) 80) in
       let len = Utf8.length str in
       let now = B.to_string b in
       Log.info (fun m -> m "deleted %S => content=%S" str now);
       if len <> 80 then
         Log.err (fun m -> m "deleted %S with length=%d instead of 80" str len);

       Lwt.return_unit
    );

  B.check b;

  let () =
(*    Theme.set_tag_prop theme L.keyword0 Props.fg_color Color.yellow;
    Theme.set_tag_prop theme L.keyword1 Props.fg_color Color.orange;
    Theme.set_tag_prop theme L.keyword2 Props.fg_color Color.cyan;
    for i = 0 to 4 do
      Theme.set_tag_prop theme  (fst L.keyword i) Props.font_desc
        (Font.font_desc  ~bold:true ~size:14 "Bitstream Vera Sans Mono");
    done;
    Theme.set_tag_prop theme L.string Props.fg_color Color.yellow;
    Theme.set_tag_prop theme L.symbol0 Props.fg_color Color.orange;
    Theme.set_tag_prop theme L.symbol1 Props.fg_color Color.red;
    Theme.set_tag_prop theme L.bcomment Props.fg_color Color.lightgreen ;
*)
(*
    Log.warn (fun m -> m "%a" Theme.pp tv1#theme);
    Log.warn (fun m -> m "%a" Theme.pp tv2#theme);
*)
    List.iter (fun (_,b) ->
       try B.set_source_language b (Some "ocaml") [@landmark "set_source_language"]
       with e ->
           Log.err (fun m -> m "%s" (Printexc.to_string e))
    )
      buffers
  in
  let len = List.length buffers in
  let _ = tv1#connect
    Widget.Key_pressed (fun ev ->
        match ev.Widget.key with
        | k when k = Tsdl.Sdl.K.escape ->
          let len = List.length buffers in
          let n = Random.int len in
          let (file, b) = List.nth buffers n in
          Log.warn (fun m -> m "switching to buffer %S" file);
          tv1#set_buffer b ;
          (*let () = tv1#print_words in*)
          true
        | _ -> false)
   in
  let%lwt () = Lwt.join [Stk.App.run ()] in
  Lwt.return_unit

let () =
  Lwt_main.run (main ())
