open Stk
open Stk.Misc

let () = let> () = Tsdl.Sdl.(init Init.(video+events))  in ()
let () = Logs.set_reporter (Misc.lwt_reporter ())

module B = Textbuffer
module Tag = Texttag.T
module Theme = Texttag.Theme
module L = Texttag.Lang

let () = Logs.set_level ~all:true (Some Logs.Warning)
let () = Logs.Src.set_level Textview.log_src (Some Logs.Warning)
(*let () = Logs.Src.set_level B.src (Some Logs.Debug)*)
let () = Logs.set_level ~all:true (Some Logs.Warning)
(*
let () = Logs.set_level ~all:true (Some Logs.Debug)
let () = Logs.Src.set_level Stk.Texture.src (Some Logs.Warning)
*)

let main () =
  let%lwt () = App.init () in
  let theme = Theme.get_or_create "default" in
  let b = B.create () in
  B.insert b 0"hello bla bla bla\nall of\nthe\nworld\n!";
  B.check b;
  let win = App.create_window ~resizable:true
    (*~rflags:Tsdl.Sdl.Renderer.software*)
    ~w:600 ~h:600 "test text" in
  let box = Pack.vbox ~pack:win#set_child () in
  let tv1 =
    let scr = Bin.scrollbox ~pack: box#pack () in
    scr#set_p Props.padding (Props.trbl__ 4) ;
    scr#set_p Props.bg_color Color.darkgreen ;
    scr#set_p Props.fill true ;
    let tv1 = Textview.textview ~buffer:b ~pack:scr#set_child () in
    tv1#set_p Textview.wrap_mode Textview.Wrap_char ;
    tv1#set_p Textview.show_line_numbers true ;
    tv1#set_p Textview.show_line_markers true ;
    let _c1 = tv1#add_cursor ~line:0 ~char: 10 () in
    let _c2 = tv1#add_cursor ~line:10 () in
    tv1
  in
  let f () =
   let c = Textbuffer.create_cursor ~offset: 20 b in
    let rec iter acc i =
      let%lwt () = Lwt_unix.sleep 0.05 in
      if i >= 200 then
        Lwt.return acc
      else
        (
         let len = Random.int 15 in
         let str = String.make len 'a' in
         Textbuffer.insert_at_cursor b c str;
         iter (len+acc) (i+1)
        )
    in
    let%lwt nb_inserted = iter 0 0 in
    prerr_endline (Printf.sprintf "buffer(%d)=%s" (Textbuffer.size b) (Textbuffer.to_string b));
    let rec iter remain =
      let%lwt () = Lwt_unix.sleep 0.05 in
      if remain > 0 then
        (
         let size = min remain (Random.int 20) in
         let p = Textbuffer.cursor_offset b c in
         let start = p - size in
         Stk.Log.warn (fun m -> m "p=%d, deleting start=%d size=%d" p start size);
         let deleted = Textbuffer.delete ~start ~size b in
         iter (remain - String.length deleted)
        )
        else
        Lwt.return_unit
    in
    let%lwt () = iter nb_inserted in
    Textbuffer.check b;
    (*Logs.Src.set_level Textview.log_src (Some Debug);
    Logs.Src.set_level Textbuffer.log_src (Some Debug);*)
    (*ignore(tv1#move_cursor ~offset: 0 ());*)
    tv1#insert "f";
    prerr_endline (tv1#text ());
    Textbuffer.check b;
    Lwt.return_unit

  in
  Lwt.async f ;
  let%lwt () = Lwt.join [Stk.App.run ()] in
  Lwt.return_unit

let () =
  Lwt_main.run (main ())
